#pragma once

#include <cstdint>
#include <type_traits>
#include <vector>

template <bool Const>
class BasicBlock {
public:
	using data_type = typename std::conditional<Const, const uint8_t, uint8_t>::type;

	explicit BasicBlock (data_type* beg);
	~BasicBlock();

	data_type* begin() { return m_begin; }
	data_type* end() { return m_begin + size(); }
	const data_type* cbegin() const { return m_begin; }
	const data_type* cend() const { return m_begin + size(); }
	const data_type* begin() const { return m_begin; }
	const data_type* end() const { return m_begin + size(); }

	static constexpr std::size_t size() { return 8192; }

	const std::vector<uint8_t>& palette() const { return m_icon_palette; }

private:
	std::vector<uint8_t> m_icon_palette;
	data_type* m_begin;
};

using Block = BasicBlock<false>;
using ConstBlock = BasicBlock<true>;
