#include "widget/block_grid.hpp"
#include "memorycard.hpp"
#include "block.hpp"
#include "icon_fetch.hpp"
#include <cassert>
#include <algorithm>

namespace duck::widget {
namespace {
	const constexpr int g_icon_spacing = 3;
} //unnamed namespace

namespace {
	int icon_at_coords (nana::point pos, int count, int columns, int w, int h, int spacing) {
		const int& x = pos.x;
		const int& y = pos.y;
		const int icon_rows = (count + (columns - 1)) / columns;
		assert(icon_rows > 0);

		std::cout << "x:" << x <<
			" y:" << y <<
			" count:" << count <<
			" col:" << columns <<
			" w:" << w <<
			" h:" << h <<
			" spacing:" << spacing <<
			" rows:" << icon_rows <<
			'\n';
		if (x < 0 or
			y < 0 or
			x >= spacing * (columns - 1) + columns * w or
			y >= spacing * (icon_rows - 1) + icon_rows * h
		) {
			return count;
		}

		const int x_icon = x / (w + spacing);
		const int y_icon = y / (h + spacing);

		std::cout << "x_icon:" << x_icon <<
			" y_icon:" << y_icon <<
			'\n';
		if ((x_icon + 1) * w <= x or (y_icon + 1) * h <= y) {
			return count;
		}

		std::cout << "ret:" << x_icon + y_icon * columns << '\n';
		return x_icon + y_icon * columns;
	}
} //unnamed namespace

namespace drawerbase::block_grid {
Trigger::Trigger() {
}

void Trigger::attached (
	nana::widget& widget,
	nana::paint::graphics& graph
) {
	m_graph = &graph;
	m_widget = &widget;

	nana::API::dev::enable_space_click(widget, true);
	nana::window wd = widget;
	nana::API::tabstop(wd);
}

void Trigger::detached() {
	m_widget = nullptr;
	m_graph = nullptr;
}

void Trigger::refresh (nana::paint::graphics& graph) {
	assert(m_widget);

	//draw background
	//graph.rectangle(true, nana::colors::black);
	if (not nana::API::dev::copy_transparent_background(*m_widget, graph))
		graph.rectangle(true, m_widget->bgcolor());

	if (m_highlight_index < m_block_count) {
		nana::rectangle r(
			m_highlight_index % m_columns * (m_icon_w + g_icon_spacing),
			m_highlight_index / m_columns * (m_icon_h + g_icon_spacing),
			m_icon_w + 2 * g_icon_spacing,
			m_icon_h + 2 * g_icon_spacing
		);
		graph.gradual_rectangle(
			r,
			nana::color(221, 215, 36),
			nana::color(29, 22, 12),
			//nana::color(246, 233, 200),
			true
		);
	}
}

void Trigger::mouse_down(nana::paint::graphics&, const nana::arg_mouse& mouse) {
}

void Trigger::mouse_enter (nana::paint::graphics& graph, const nana::arg_mouse& mouse) {
	m_highlight_index = icon_at_coords(
		mouse.pos,
		static_cast<int>(m_block_count),
		m_columns,
		m_icon_w,
		m_icon_h,
		g_icon_spacing
	);
	std::cout << "mouse enter, selected " << m_highlight_index << '\n';
	refresh(graph);
}

void Trigger::mouse_leave (nana::paint::graphics& graph, const nana::arg_mouse& mouse) {
	m_highlight_index = m_block_count;
	std::cout << "mouse leaving\n";
	refresh(graph);
}

void Trigger::set_block_count (std::size_t sz) noexcept {
	if (m_highlight_index >= m_block_count)
		m_highlight_index = sz;
	else if (sz < m_block_count)
		m_highlight_index = std::min(m_highlight_index, sz);

	m_block_count = sz;
}

void Trigger::set_icon_size (nana::size sz) noexcept {
	m_icon_w = sz.width;
	m_icon_h = sz.height;
}

void Trigger::set_columns(int c) noexcept {
	m_columns = c;
}

int Trigger::columns() const noexcept {
	return m_columns;
}
} //namespace drawerbase::block_grid

BlockGrid::BlockGrid (
	nana::window wd,
	unsigned int columns,
	bool visible
) {
	this->create(wd, nana::rectangle{}, visible);
	get_drawer_trigger().set_columns(columns);
}

void BlockGrid::emplace_back (AnimationWithSize&& anim) {
	const int columns = get_drawer_trigger().columns();
	const int count = static_cast<int>(m_blocks.size());
	const constexpr int s = g_icon_spacing;
	get_drawer_trigger().set_icon_size(anim.size());
	const int w = anim.size().width;
	const int h = anim.size().height;
	const int x = (count % columns) * (s + w) + s;
	const int y = (count / columns) * (s + h) + s;

	m_blocks.push_back(std::move(anim));
	m_blocks.back().output(*this, nana::point(x, y));

	get_drawer_trigger().set_block_count(m_blocks.size());
}
} //namespace duck::widget
